@@ -1,6 +1,4 @@
-"""
-Parses Polarion docstrings.
-"""
+"""Parse Polarion docstrings."""
 
 import ast
 from collections import namedtuple
@@ -13,7 +11,7 @@ ErrorRecord = namedtuple("ErrorRecord", "lineno column type message")
 
 
 def get_section_start(doc_list, section):
-    """Finds the line with "section" (e.g. "Polarion", "testSteps", etc.)."""
+    """Find the line with "section" (e.g. "Polarion", "testSteps", etc.)."""
     section = "{}:".format(section)
     for index, line in enumerate(doc_list):
         if section != line.strip():
@@ -23,8 +21,16 @@ def get_section_start(doc_list, section):
     return None, None
 
 
+def items_str_to_list(items):
+    """Split items separated by space or comma to list."""
+    sep = "," if "," in items else " "
+    items_list = [item.strip('" ') for item in items.split(sep)]
+    items_list = [item for item in items_list if item]
+    return items_list
+
+
 def add_errors(docstring_dict, errors):
-    """Adds new `ErrorRecord`(s) to `_errors_parsing`."""
+    """Add new `ErrorRecord`(s) to `_errors_parsing`."""
     if not errors:
         return
 
@@ -53,6 +59,7 @@ class DOCSTRING_SECTIONS:  # noqa
     polarion = "Polarion"
     steps = "testSteps"
     results = "expectedResults"
+    linked_items = "linkedWorkItems"
 
 
 class DocstringParser:
@@ -65,7 +72,7 @@ class DocstringParser:
 
     @staticmethod
     def _get_key_value(line):
-        """Gets the key and value out of docstring line."""
+        """Get the key and value out of docstring line."""
         data = line.split(":")
         if len(data) == 1:
             data.append("")
@@ -73,14 +80,41 @@ class DocstringParser:
         key = data[0].strip()
 
         value = ":".join(data[1:]).strip()
-        if value.lower() in ("none", "null"):
+        value_lower = value.lower()
+        if value_lower in ("none", "null"):
             value = None
+        elif value_lower == "true":
+            value = True
+        elif value_lower == "false":
+            value = False
 
         return key, value
 
+    @staticmethod
+    def _is_first_multiline(line_stripped):
+        """Check if the current line is a first line of multiline string."""
+        is_first_multiline = line_stripped[-1] == ">"
+        is_first_multiline = is_first_multiline and len(line_stripped.split(" ")) <= 2
+        return is_first_multiline
+
+    @staticmethod
+    def _append_to_prev(line_stripped, curr_indent, indent, is_multiline):
+        """Check if the current line should be appended to previous line."""
+        if curr_indent <= indent:
+            return False
+        # the line is either part of multiline string...
+        if is_multiline:
+            return True
+        # or is not in "keywod: value" format
+        first_word = line_stripped.split(" ")[0] or line_stripped
+        is_keyword = first_word[-1] == ":"
+        if not is_keyword:
+            return True
+        return False
+
     # pylint: disable=too-many-locals
     def lines_to_dict(self, lines, start=0, lineno_offset=0, stop=None):
-        """Creates dictionary out of docstring lines.
+        """Create dictionary out of docstring lines.
 
         Includes column and line number info for each record.
         """
@@ -90,7 +124,8 @@ class DocstringParser:
             lines = lines[start:]
 
         docstring_dict = {}
-        indent = len(lines[0]) - len(lines[0].lstrip(" "))
+        indent = 0
+        multiline_indent = 0
         is_multiline = False
         prev_key = None
         for num, line in enumerate(lines, 1):
@@ -100,23 +135,25 @@ class DocstringParser:
 
             curr_indent = len(line) - len(line.lstrip(" "))
 
+            if not indent:
+                indent = curr_indent
+
             if curr_indent < indent:
                 break
 
             # check if the line should be appended to previous key
-            is_first_multiline = False
-            if not is_multiline:
-                is_first_multiline = line_stripped[-1] == ">"
-                is_first_multiline = is_first_multiline and len(line_stripped.split(" ")) <= 2
-                if is_first_multiline:
-                    # we might want to append this very line to `prev_key` if it's in a list
-                    is_multiline = True
-                    # remove the ">"
-                    line_stripped = line_stripped[:-1].strip()
+            is_multiline = is_multiline and curr_indent > multiline_indent
+            is_first_multiline = not is_multiline and self._is_first_multiline(line_stripped)
+            if is_first_multiline:
+                # we want to append this line to `docstring_dict[prev_key]` if
+                # it's part of a list, e.g. testSteps
+                is_multiline = True
+                # store current indent level
+                multiline_indent = curr_indent
+                # remove the ">"
+                line_stripped = line_stripped[:-1].strip()
 
-            first_word = line_stripped.split(" ")[0] or line_stripped
-            is_keyword = first_word[-1] == ":"
-            if prev_key and curr_indent > indent and (not is_keyword or is_multiline):
+            if prev_key and self._append_to_prev(line_stripped, curr_indent, indent, is_multiline):
                 prev_lineno, prev_indent, prev_value = docstring_dict[prev_key]
                 sep = ""
                 if prev_value:
@@ -144,14 +181,14 @@ class DocstringParser:
                 continue
 
             key, value = self._get_key_value(line_stripped)
-            docstring_dict[key] = ValueRecord(num + lineno_offset, indent, value.strip())
+            docstring_dict[key] = ValueRecord(num + lineno_offset, indent, value)
             prev_key = key
 
         return docstring_dict
 
-    @staticmethod
-    def lines_to_list(lines, start=0, lineno_offset=0, stop=None):
-        """Creates list out of docstring lines.
+    @classmethod
+    def lines_to_list(cls, lines, start=0, lineno_offset=0, stop=None):
+        """Create list out of docstring lines.
 
         Includes column and line number info for each line.
         """
@@ -161,28 +198,32 @@ class DocstringParser:
             lines = lines[start:]
 
         lines_list = []
+        indent = 0
+        multiline_indent = 0
         is_multiline = False
-        indent = len(lines[0]) - len(lines[0].lstrip(" "))
         for num, line in enumerate(lines, 1):
             line_stripped = line.strip()
             if not line_stripped:
                 continue
+
             curr_indent = len(line) - len(line.lstrip(" "))
+
+            if not indent:
+                indent = curr_indent
+
             if curr_indent < indent:
                 break
 
             # check if the line should be appended to previous key
-            is_first_multiline = False
-            if not is_multiline:
-                is_first_multiline = line_stripped[-1] == ">"
-                is_first_multiline = is_first_multiline and len(line_stripped.split(" ")) <= 2
-                if is_first_multiline:
-                    # remove the ">"
-                    line_stripped = line_stripped[:-1].strip()
+            is_multiline = is_multiline and curr_indent > multiline_indent
+            is_first_multiline = not is_multiline and cls._is_first_multiline(line_stripped)
+            if is_first_multiline:
+                # store current indent level
+                multiline_indent = curr_indent
+                # remove the ">"
+                line_stripped = line_stripped[:-1].strip()
 
-            first_word = line_stripped.split(" ")[0] or line_stripped
-            is_keyword = first_word[-1] == ":"
-            if curr_indent > indent and (not is_keyword or is_multiline):
+            if cls._append_to_prev(line_stripped, curr_indent, indent, is_multiline):
                 prev_lineno, prev_indent, prev_value = lines_list.pop()
                 lines_list.append(
                     ValueRecord(prev_lineno, prev_indent, "{} {}".format(prev_value, line_stripped))
@@ -200,7 +241,7 @@ class DocstringParser:
         return lines_list
 
     def strip_polarion_data(self):
-        """Removes the Polarion section from the docstring."""
+        """Remove the Polarion section from the docstring."""
         docstring_list = self.docstring.split("\n")
         new_docstring_list = []
         polarion_section = "{}:".format(self.SECTIONS.polarion)
@@ -208,27 +249,29 @@ class DocstringParser:
         in_polarion = False
 
         for line in docstring_list:
+            # find beginning of Polarion section
             if line.strip() == polarion_section:
                 indent = len(line) - len(line.lstrip(" "))
                 in_polarion = True
                 continue
-            if indent:
+            # skip blank lines and lines more indented than the beginning of
+            # Polarion section
+            if in_polarion:
+                if not line.strip():
+                    continue
                 curr_indent = len(line) - len(line.lstrip(" "))
                 if curr_indent > indent:
                     continue
-                # If this is the next line after Polarion section,
-                # don't save blank line.
-                if in_polarion:
-                    in_polarion = False
-                    if not line.strip():
-                        continue
-            new_docstring_list.append(line)
+                in_polarion = False
+            new_docstring_list.append(line.rstrip())
 
         new_docstring = "\n".join(new_docstring_list)
-        return new_docstring.strip()
+        return new_docstring.strip("\n")
 
     def parse(self):
-        """Parses docstring to dictionary. E.g.:
+        """Parse docstring to dictionary.
+
+        E.g.:
 
         Polarion:
             assignee: mkourim
@@ -274,6 +317,12 @@ class DocstringParser:
                 )
                 docstring_dict[self.SECTIONS.results] = results_list
 
+        linked_items = docstring_dict.get(self.SECTIONS.linked_items)
+        if linked_items and linked_items.value:
+            docstring_dict[self.SECTIONS.linked_items] = ValueRecord(
+                linked_items.lineno, linked_items.column, items_str_to_list(linked_items.value)
+            )
+
         return docstring_dict
 
 
@@ -289,7 +338,7 @@ class FileParser:
         self.tree = tree or self.get_tree()
 
     def get_tree(self):
-        """Returns ast tree."""
+        """Return ast tree."""
         with open(self.filename) as infile:
             source = infile.read()
 
@@ -298,7 +347,7 @@ class FileParser:
 
     @staticmethod
     def get_docstring_from_node(node):
-        """Gets docstring from ast node."""
+        """Get docstring from ast node."""
         docstring = None
         body = getattr(node, "body", None)
         body = body[0] if body else node
@@ -310,12 +359,6 @@ class FileParser:
         except Exception:
             return None
 
-        # for Python2
-        try:
-            docstring = docstring.decode("utf-8")
-        except AttributeError:
-            pass
-
         return docstring
 
     def _get_nodeid(self, node_name, class_name):
@@ -328,7 +371,7 @@ class FileParser:
         return nodeid
 
     def process_docstring(self, docstring, node, nodeid=None, level=LEVELS.function):
-        """Returns parsed Polarion docstring."""
+        """Return parsed Polarion docstring."""
         body = getattr(node, "body", None)
         body = body[0] if body else node
 
@@ -367,13 +410,13 @@ class FileParser:
         )
 
     def process_node(self, node, class_name, level):
-        """Returns parsed Polarion docstring present in the function."""
+        """Return parsed Polarion docstring present in the function."""
         docstring = self.get_docstring_from_node(node)
         nodeid = self._get_nodeid(getattr(node, "name", None), class_name)
         return self.process_docstring(docstring, node, nodeid, level)
 
     def process_ast_body(self, body, class_name=None, is_top=False):
-        """Recursively iterates over specified part of ast tree to process functions."""
+        """Recursively iterate over specified part of ast tree to process functions."""
         docstrings = {}
         if is_top:
             docstring = self.process_node(body[0], None, LEVELS.file)
@@ -401,7 +444,7 @@ class FileParser:
         return docstrings
 
     def get_docstrings(self):
-        """Returns parsed Polarion docstrings present in the python source file."""
+        """Return parsed Polarion docstrings present in the python source file."""
         return self.process_ast_body(self.tree.body, is_top=True)
 
 
@@ -409,17 +452,17 @@ class FileParser:
 
 
 def parse_docstring(docstring):
-    """Parses docstring to dictionary."""
+    """Parse docstring to dictionary."""
     return DocstringParser(docstring).parse()
 
 
 def strip_polarion_data(docstring):
-    """Removes the Polarion section from the docstring."""
+    """Remove the Polarion section from the docstring."""
     return DocstringParser(docstring).strip_polarion_data()
 
 
 def get_docstrings_in_file(filename, tree=None, tests_only=True):
-    """Returns parsed Polarion docstrings present in the python source file."""
+    """Return parsed Polarion docstrings present in the python source file."""
     return FileParser(filename, tree=tree, tests_only=tests_only).get_docstrings()
 
 
@@ -447,7 +490,7 @@ def _merge_components(docstrings_dict, nodeid):
 
 
 def merge_docstrings(docstrings):
-    """Merges docstrings with their parents."""
+    """Merge docstrings with their parents."""
     docstrings_dict = {}
 
     # create dict out of records
@@ -468,7 +511,7 @@ def merge_docstrings(docstrings):
 
 
 def merge_record(record, docstrings_dict, nodeid=None):
-    """Merges single record with its parents based on nodeid."""
+    """Merge single record with its parents based on nodeid."""
     merged_record = {}
 
     if isinstance(record, DocstringRecord):
