@@ -20,6 +20,11 @@ def source_file_merge():
     return os.path.join(TESTS_DATA_DIR, "docstrings_merge.py")
 
 
+@pytest.fixture(scope="session")
+def source_file_empty():
+    return os.path.join(TESTS_DATA_DIR, "docstrings_empty.py")
+
+
 @pytest.fixture(scope="session")
 def config():
     with open(CONFIG_TEMPLATE, encoding="utf-8") as config_file:
