@@ -4,7 +4,7 @@ from polarion_docstrings import checker
 
 EXPECTED = [
     (6, 4, 'P666 Unknown field "foo"'),
-    (17, 8, 'P667 Invalid value "huge" of the "caseimportance" field'),
+    (17, 8, 'P667 Invalid value(s) of the "caseimportance" field: huge'),
     (18, 8, 'P666 Unknown field "bar"'),
     (
         19,
